<?php
if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'ewduaspSettings' ) ) {
/**
 * Class to handle configurable settings for Ultimate Appointment Scheduling
 * @since 2.0.0
 */
class ewduaspSettings {

	public $currency_options = array();

	public $language_options = array();

	public $hour_options = array();

	public $email_options = array();

	/**
	 * Default values for settings
	 * @since 2.0.0
	 */
	public $defaults = array();

	/**
	 * Stored values for settings
	 * @since 2.0.0
	 */
	public $settings = array();

	public function __construct() {

		add_action( 'init', array( $this, 'set_defaults' ) );

		add_action( 'init', array( $this, 'set_field_options' ) );

		add_action( 'init', array( $this, 'load_settings_panel' ) );
	}

	/**
	 * Load the plugin's default settings
	 * @since 2.0.0
	 */
	public function set_defaults() {

		$this->defaults = array(

			'date-format'					=> 'Y-m-d H:i:s',
			'required-information'			=> array(),

			'maximum-days-advance'			=> 3650,

			'booking-form-style'			=> 'standard',
			'access-role'					=> 'manage_options',

			'custom-fields'					=> array(),

			'paypal-prepayment'				=> 'none',
			'currency-code'					=> 'USD',

			'reminders-cache-time'			=> 10,

			'email-messages'				=> array(),

			'label-any'						=> __( 'Any', 'ultimate-appointment-scheduling' ),
			'label-thank-you-submit'		=> __( 'Thank you, your appointment has been successfully created.', 'ultimate-appointment-scheduling' ),
		);

		$this->defaults = apply_filters( 'ewd_uasp_defaults', $this->defaults );
	}

	/**
	 * Put all of the available possible select options into key => value arrays
	 * @since 2.0.0
	 */
	public function set_field_options() {

		$this->currency_options = array(
			'AUD' => __( 'Australian Dollar', 'ultimate-appointment-scheduling'),
			'BRL' => __( 'Brazilian Real', 'ultimate-appointment-scheduling'),
			'CAD' => __( 'Canadian Dollar', 'ultimate-appointment-scheduling'),
			'CZK' => __( 'Czech Koruna', 'ultimate-appointment-scheduling'),
			'DKK' => __( 'Danish Krone', 'ultimate-appointment-scheduling'),
			'EUR' => __( 'Euro', 'ultimate-appointment-scheduling'),
			'HKD' => __( 'Hong Kong Dollar', 'ultimate-appointment-scheduling'),
			'HUF' => __( 'Hungarian Forint', 'ultimate-appointment-scheduling'),
			'ILS' => __( 'Israeli New Sheqel', 'ultimate-appointment-scheduling'),
			'JPY' => __( 'Japanese Yen', 'ultimate-appointment-scheduling'),
			'MYR' => __( 'Malaysian Ringgit', 'ultimate-appointment-scheduling'),
			'MXN' => __( 'Mexican Peso', 'ultimate-appointment-scheduling'),
			'NOK' => __( 'Norwegian Krone', 'ultimate-appointment-scheduling'),
			'NZD' => __( 'New Zealand Dollar', 'ultimate-appointment-scheduling'),
			'PHP' => __( 'Philippine Peso', 'ultimate-appointment-scheduling'),
			'PLN' => __( 'Polish Zloty', 'ultimate-appointment-scheduling'),
			'GBP' => __( 'Pound Sterling', 'ultimate-appointment-scheduling'),
			'RUB' => __( 'Russian Ruble', 'ultimate-appointment-scheduling'),
			'SGD' => __( 'Singapore Dollar', 'ultimate-appointment-scheduling'),
			'SEK' => __( 'Swedish Krona', 'ultimate-appointment-scheduling'),
			'CHF' => __( 'Swiss Franc', 'ultimate-appointment-scheduling'),
			'TWD' => __( 'Taiwan New Dollar', 'ultimate-appointment-scheduling'),
			'THB' => __( 'Thai Baht', 'ultimate-appointment-scheduling'),
			'TRY' => __( 'Turkish Lira', 'ultimate-appointment-scheduling'),
			'USD' => __( 'U.S. Dollar', 'ultimate-appointment-scheduling'),
		);

		$this->language_options = array(
			'en' 		=> __( 'English', 'ultimate-appointment-scheduling' ),
			'af' 		=> __( 'Afrikaans', 'ultimate-appointment-scheduling' ),
			'ar' 		=> __( 'Arabic', 'ultimate-appointment-scheduling' ),
			'ar-dz' 	=> __( 'Arabic (Algeria)', 'ultimate-appointment-scheduling' ),
			'ar-kw' 	=> __( 'Arabic (Kuwait)', 'ultimate-appointment-scheduling' ),
			'ar-ly' 	=> __( 'Arabic (Libya)', 'ultimate-appointment-scheduling' ),
			'ar-ma' 	=> __( 'Arabic (Morocco)', 'ultimate-appointment-scheduling' ),
			'ar-sa' 	=> __( 'Arabic (Saudi Arabia)', 'ultimate-appointment-scheduling' ),
			'ar-tn' 	=> __( 'Arabic (Tunisia)', 'ultimate-appointment-scheduling' ),
			'bg' 		=> __( 'Bulgarian', 'ultimate-appointment-scheduling' ),
			'ca' 		=> __( 'Catalan', 'ultimate-appointment-scheduling' ),
			'cs' 		=> __( 'Czech', 'ultimate-appointment-scheduling' ),
			'da' 		=> __( 'Danish', 'ultimate-appointment-scheduling' ),
			'de' 		=> __( 'German', 'ultimate-appointment-scheduling' ),
			'de-at' 	=> __( 'German (Austria)', 'ultimate-appointment-scheduling' ),
			'de-ch' 	=> __( 'German (Switzerland)', 'ultimate-appointment-scheduling' ),
			'el' 		=> __( 'Greek', 'ultimate-appointment-scheduling' ),
			'en-au' 	=> __( 'English (Australia)', 'ultimate-appointment-scheduling' ),
			'en-ca' 	=> __( 'English (Canada)', 'ultimate-appointment-scheduling' ),
			'en-gb' 	=> __( 'English (Great Britain)', 'ultimate-appointment-scheduling' ),
			'en-ie' 	=> __( 'English (Ireland)', 'ultimate-appointment-scheduling' ),
			'en-nz' 	=> __( 'English (New Zealand)', 'ultimate-appointment-scheduling' ),
			'es' 		=> __( 'Spanish', 'ultimate-appointment-scheduling' ),
			'es-do' 	=> __( 'Spanish (Dominican Republic)', 'ultimate-appointment-scheduling' ),
			'es-us' 	=> __( 'Spanish (United States)', 'ultimate-appointment-scheduling' ),
			'et' 		=> __( 'Estonian', 'ultimate-appointment-scheduling' ),
			'eu' 		=> __( 'Basque', 'ultimate-appointment-scheduling' ),
			'fa' 		=> __( 'Persian (Farsi)', 'ultimate-appointment-scheduling' ),
			'fi' 		=> __( 'Finnish', 'ultimate-appointment-scheduling' ),
			'fr' 		=> __( 'French', 'ultimate-appointment-scheduling' ),
			'fr-ca' 	=> __( 'French (Canada)', 'ultimate-appointment-scheduling' ),
			'fr-ch' 	=> __( 'French (Switzerland)', 'ultimate-appointment-scheduling' ),
			'gl' 		=> __( 'Galician', 'ultimate-appointment-scheduling' ),
			'he' 		=> __( 'Hebrew', 'ultimate-appointment-scheduling' ),
			'hi' 		=> __( 'Hindi', 'ultimate-appointment-scheduling' ),
			'hr' 		=> __( 'Croatian', 'ultimate-appointment-scheduling' ),
			'hu' 		=> __( 'Hungarian', 'ultimate-appointment-scheduling' ),
			'id' 		=> __( 'Indonesian', 'ultimate-appointment-scheduling' ),
			'is' 		=> __( 'Icelandic', 'ultimate-appointment-scheduling' ),
			'it' 		=> __( 'Italian', 'ultimate-appointment-scheduling' ),
			'ja' 		=> __( 'Japanese', 'ultimate-appointment-scheduling' ),
			'kk' 		=> __( 'Kazakh', 'ultimate-appointment-scheduling' ),
			'ko' 		=> __( 'Korean', 'ultimate-appointment-scheduling' ),
			'lb' 		=> __( 'Luxembourgish', 'ultimate-appointment-scheduling' ),
			'lt' 		=> __( 'Lithuanian', 'ultimate-appointment-scheduling' ),
			'lv' 		=> __( 'Latvian (Lettish)', 'ultimate-appointment-scheduling' ),
			'mk' 		=> __( 'Macedonian', 'ultimate-appointment-scheduling' ),
			'ms' 		=> __( 'Malay', 'ultimate-appointment-scheduling' ),
			'ms-my' 	=> __( 'Malay (Malaysia)', 'ultimate-appointment-scheduling' ),
			'nb' 		=> __( 'Norwegian bokmål', 'ultimate-appointment-scheduling' ),
			'nl' 		=> __( 'Dutch', 'ultimate-appointment-scheduling' ),
			'nl-be' 	=> __( 'Dutch (Belgium)', 'ultimate-appointment-scheduling' ),
			'nn' 		=> __( 'Norwegian nynorsk', 'ultimate-appointment-scheduling' ),
			'pl' 		=> __( 'Polish', 'ultimate-appointment-scheduling' ),
			'pt' 		=> __( 'Portuguese', 'ultimate-appointment-scheduling' ),
			'pt-br' 	=> __( 'Portuguese (Brazil)', 'ultimate-appointment-scheduling' ),
			'ro' 		=> __( 'Romanian', 'ultimate-appointment-scheduling' ),
			'ru' 		=> __( 'Russian', 'ultimate-appointment-scheduling' ),
			'sk' 		=> __( 'Slovak', 'ultimate-appointment-scheduling' ),
			'sl' 		=> __( 'Slovenian', 'ultimate-appointment-scheduling' ),
			'sq' 		=> __( 'Albanian', 'ultimate-appointment-scheduling' ),
			'sr' 		=> __( 'Serbian', 'ultimate-appointment-scheduling' ),
			'sr-cyrl' 	=> __( 'Serbian (Cyrillic)', 'ultimate-appointment-scheduling' ),
			'sv' 		=> __( 'Swedish', 'ultimate-appointment-scheduling' ),
			'th' 		=> __( 'Thai', 'ultimate-appointment-scheduling' ),
			'tr' 		=> __( 'Turkish', 'ultimate-appointment-scheduling' ),
			'uk' 		=> __( 'Ukrainian', 'ultimate-appointment-scheduling' ),
			'vi' 		=> __( 'Vietnamese', 'ultimate-appointment-scheduling' ),
			'zh-cn' 	=> __( 'Chinese', 'ultimate-appointment-scheduling' ),
			'zh-tw' 	=> __( 'Chinese (Taiwan)', 'ultimate-appointment-scheduling' ),
		);

		for ( $i = 0; $i < 24; $i++ ) {

			$this->hour_options[ $i ] = $i . ':00';
		}

		$emails = ewd_uasp_decode_infinite_table_setting( $this->get_setting( 'email-messages-array' ) );

		foreach ( $emails as $email ) { 

			$this->email_options[ $email->id ] = $email->name;
		}

		if ( in_array( 'ultimate-wp-mail/Main.php', (array) get_option( 'active_plugins' ) ) ) {

			$this->email_options[-1] = '';
			
			$args = array(
				'post_type'		=> 'uwpm_mail_template',
				'numberposts'	=> -1
			);

			$uwpm_emails = get_posts( $args );

			foreach ( $uwpm_emails as $uwpm_email ) { 

				$email_id = $uwpm_email->ID * -1;

				$this->email_options[ $email_id ] = $uwpm_email->post_title;
			}
		}
	}

	/**
	 * Get a setting's value or fallback to a default if one exists
	 * @since 2.0.0
	 */
	public function get_setting( $setting ) { 

		if ( empty( $this->settings ) ) {
			$this->settings = get_option( 'ewd-uasp-settings' );
		}
		
		if ( ! empty( $this->settings[ $setting ] ) ) {
			return apply_filters( 'ewd-uasp-settings-' . $setting, $this->settings[ $setting ] );
		}

		if ( ! empty( $this->defaults[ $setting ] ) ) { 
			return apply_filters( 'ewd-uasp-settings-' . $setting, $this->defaults[ $setting ] );
		}

		return apply_filters( 'ewd-uasp-settings-' . $setting, null );
	}

	/**
	 * Set a setting to a particular value
	 * @since 2.0.0
	 */
	public function set_setting( $setting, $value ) {

		$this->settings[ $setting ] = $value;
	}

	/**
	 * Save all settings, to be used with set_setting
	 * @since 2.0.0
	 */
	public function save_settings() {
		
		update_option( 'ewd-uasp-settings', $this->settings );
	}

	/**
	 * Load the admin settings page
	 * @since 2.0.0
	 * @sa https://github.com/NateWr/simple-admin-pages
	 */
	public function load_settings_panel() {

		global $ewd_uasp_controller;

		require_once( EWD_UASP_PLUGIN_DIR . '/lib/simple-admin-pages/simple-admin-pages.php' );
		$sap = sap_initialize_library(
			$args = array(
				'version'       => '2.5.1',
				'lib_url'       => EWD_UASP_PLUGIN_URL . '/lib/simple-admin-pages/',
			)
		);
		
		$sap->add_page(
			'submenu',
			array(
				'id'            => 'ewd-uasp-settings',
				'title'         => __( 'Settings', 'ultimate-appointment-scheduling' ),
				'menu_title'    => __( 'Settings', 'ultimate-appointment-scheduling' ),
				'parent_menu'	=> 'ewd-uasp-appointments',
				'description'   => '',
				'capability'    => $this->get_setting( 'access-role' ),
				'default_tab'   => 'ewd-uasp-basic-tab',
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-basic-tab',
				'title'         => __( 'Basic', 'ultimate-appointment-scheduling' ),
				'is_tab'		=> true,
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-general',
				'title'         => __( 'General', 'ultimate-appointment-scheduling' ),
				'tab'	        => 'ewd-uasp-basic-tab',
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'warningtip',
			array(
				'id'			=> 'shortcodes-reminder',
				'title'			=> __( 'REMINDER:', 'ultimate-appointment-scheduling' ),
				'placeholder'	=> __( 'REMINDERS NEED TO GO HERE' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'textarea',
			array(
				'id'			=> 'custom-css',
				'title'			=> __( 'Custom CSS', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'You can add custom CSS styles to your appointment booking page in the box above.', 'ultimate-appointment-scheduling' ),			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'toggle',
			array(
				'id'			=> 'disable-appointment-editing',
				'title'			=> __( 'Disable Appointment Editing', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Should guests be blocked from editing appointments that have been previously made?', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'toggle',
			array(
				'id'			=> 'multi-step-booking',
				'title'			=> __( 'Multi-Step Booking', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Should booking an appointment be split into multiple steps or happen all in one place?', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'text',
			array(
				'id'            => 'time-between-appointments',
				'title'         => __( 'Time Between Appointments', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'How much time should there be between scheduled appointments? (in minutes)', 'ultimate-appointment-scheduling' ),
				'small'			=> true
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'checkbox',
			array(
				'id'			=> 'required-information',
				'title'			=> __( 'Required Information', 'ultimate-appointment-scheduling' ),
				'description'	=> '',
				'options'		=> array(
					'name' 			=> 'Name',
					'phone'			=> 'Phone',
					'email'			=> 'Email',
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'select',
			array(
				'id'            => 'hours-format',
				'title'         => __( 'Hours Format', 'ultimate-appointment-scheduling' ),
				'description'   => '', 
				'blank_option'	=> false,
				'options'       => array(
					'24'			=> __( '24 Hour', 'ultimate-appointment-scheduling' ),
					'12'			=> __( '12 Hour', 'ultimate-appointment-scheduling' ),
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'text',
			array(
				'id'            => 'date-format',
				'title'         => __( 'PHP Date Format', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Use this field to specify a PHP date format to be used when the date is included (e.g. in emails). More info about PHP date formats can be found <a href="https://secure.php.net/manual/en/function.date.php" target="_blank">here</a>.', 'ultimate-appointment-scheduling' ),
				'small'			=> true
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'select',
			array(
				'id'            => 'client-email-details',
				'title'         => __( 'Client Email Details', 'ultimate-appointment-scheduling' ),
				'description'   => __( 'What email, if any, should be sent to clients when they book an appointment?', 'ultimate-appointment-scheduling' ), 
				'blank_option'	=> true,
				'options'       => $this->email_options
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'count',
			array(
				'id'			=> 'minimum-days-advance',
				'title'			=> __( 'Minimum Days in Advance', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'What is the minimum number of days in advance an appointment can be booked? <strong>(Leave blank if using the Minimum Hours setting below.)</strong>', 'ultimate-appointment-scheduling' ),
				'blank_option'	=> true,
				'min_value'		=> 1,
				'max_value'		=> 100,
				'increment'		=> 1,
				'units'			=> array( 'days' => 'Days' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'count',
			array(
				'id'			=> 'minimum-hours-advance',
				'title'			=> __( 'Minimum Hours in Advance', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'What is the minimum number of hours in advance an appointment can be booked? <strong>(Leave blank if using the Minimum Days setting above.)</strong>', 'ultimate-appointment-scheduling' ),
				'blank_option'	=> true,
				'min_value'		=> 1,
				'max_value'		=> 48,
				'increment'		=> 1,
				'units'			=> array( 'hours' => 'Hours' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'count',
			array(
				'id'			=> 'maximum-days-advance',
				'title'			=> __( 'Maximum Days in Advance', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'What is the maximum number of days in advance an appointment can be booked?', 'ultimate-appointment-scheduling' ),
				'blank_option'	=> false,
				'min_value'		=> 1,
				'max_value'		=> 365,
				'increment'		=> 1,
				'units'			=> array( 'days' => 'Days' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'select',
			array(
				'id'            => 'calendar-starting-layout',
				'title'         => __( 'Calendar Starting Layout', 'ultimate-appointment-scheduling' ),
				'description'   => __( 'What layout should the calendar start in?', 'ultimate-appointment-scheduling' ), 
				'blank_option'	=> false,
				'options'       => array(
					'agendaDay'		=> __( 'Day', 'ultimate-appointment-scheduling' ),
					'agendaWeek'	=> __( 'Week', 'ultimate-appointment-scheduling' ),
					'month'			=> __( 'Month', 'ultimate-appointment-scheduling' ),
					'listWeek'		=> __( 'List', 'ultimate-appointment-scheduling' ),
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'select',
			array(
				'id'            => 'calendar-starting-time',
				'title'         => __( 'Calendar Starting Hour', 'ultimate-appointment-scheduling' ),
				'description'   => __( 'What should the default first/top time be in the calendar (e.g. if you\'re in the daily view)?', 'ultimate-appointment-scheduling' ), 
				'blank_option'	=> false,
				'options'       => $this->hour_options
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'count',
			array(
				'id'			=> 'calendar-offset',
				'title'			=> __( 'Calendar Offset', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Use this option so specify how far ahead (in days, weeks or months) the default opening date of the calendar will be', 'ultimate-appointment-scheduling' ),
				'blank_option'	=> false,
				'min_value'		=> 1,
				'max_value'		=> 30,
				'increment'		=> 1,
				'units'			=> array( 
					'offsetDay' 	=> 'Days', 
					'offsetWeek' 	=> 'Weeks', 
					'offsetMonth' 	=> 'Months', 
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-general',
			'select',
			array(
				'id'            => 'calendar-language',
				'title'         => __( 'Calendar Language', 'ultimate-appointment-scheduling' ),
				'description'   => __( 'Select the language of the calendar in the calendar style booking form.', 'ultimate-appointment-scheduling' ), 
				'blank_option'	=> false,
				'options'       => $this->language_options
			)
		);

		if ( ! $ewd_uasp_controller->permissions->check_permission( 'premium' ) ) {
			$ewd_uasp_premium_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> '#',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-appointment-scheduling/'
			);
		}
		else { $ewd_uasp_premium_permissions = array(); }

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-premium-tab',
				'title'         => __( 'Premium', 'ultimate-appointment-scheduling' ),
				'is_tab'		=> true,
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-premium-general',
					'title'         => __( 'General', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-premium-tab',
				),
				$ewd_uasp_premium_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-premium-general',
			'radio',
			array(
				'id'			=> 'booking-form-style',
				'title'			=> __( 'Booking Form Style', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'What style should the booking form use?', 'ultimate-appointment-scheduling' ),
				'options'		=> array(
					'standard'		=> __( 'Standard', 'ultimate-appointment-scheduling' ),
					'contemporary'	=> __( 'Contemporary', 'ultimate-appointment-scheduling' ),
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-premium-general',
			'select',
			array(
				'id'            => 'admin-appointment-notification',
				'title'         => __( 'Admin Appointment Notification', 'ultimate-appointment-scheduling' ),
				'description'   => __( 'What email, if any, should be sent to the administrator when an appointment is booked?', 'ultimate-appointment-scheduling' ), 
				'blank_option'	=> true,
				'options'       => $this->email_options
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-premium-general',
			'text',
			array(
				'id'            => 'admin-email-address',
				'title'         => __( 'Admin Email Address', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Provide the email address at which you would like to receive the admin email notification chosen in the option above.', 'ultimate-appointment-scheduling' ),
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-premium-general',
			'select',
			array(
				'id'            => 'service-provider-notification',
				'title'         => __( 'Service Provider Appointment Notification', 'ultimate-appointment-scheduling' ),
				'description'   => __( 'What email, if any, should be sent to the service provider when an appointment is booked?', 'ultimate-appointment-scheduling' ), 
				'blank_option'	=> true,
				'options'       => $this->email_options
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-premium-general',
			'toggle',
			array(
				'id'			=> 'use-captcha',
				'title'			=> __( 'Require Captcha', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Should a captcha field be added to your appointment scheduling form?', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-premium-general',
			'select',
			array(
				'id'            => 'access-role',
				'title'         => __( 'Set Access Role', 'ultimate-appointment-scheduling' ),
				'description'   => __( 'Who should have access to the "Appointments" admin menu?', 'ultimate-appointment-scheduling' ), 
				'blank_option'	=> false,
				'options'       => array(
					'administrator'				=> __( 'Administrator', 'ultimate-appointment-scheduling' ),
					'delete_others_pages'		=> __( 'Editor', 'ultimate-appointment-scheduling' ),
					'delete_published_posts'	=> __( 'Author', 'ultimate-appointment-scheduling' ),
					'delete_posts'				=> __( 'Contributor', 'ultimate-appointment-scheduling' ),
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-premium-general',
			'toggle',
			array(
				'id'			=> 'require-login',
				'title'			=> __( 'Require Login', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Do visitors have to login before they can book an appointment?', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-premium-general',
			'text',
			array(
				'id'            => 'wordpress-login-url',
				'title'         => __( 'WordPress Login URL', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'The URL of your WordPress login page, if different from the expected page.', 'ultimate-appointment-scheduling' ),
			)
		);

		if ( ! $ewd_uasp_controller->permissions->check_permission( 'custom_fields' ) ) {
			$ewd_uasp_custom_fields_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> '#',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-appointment-scheduling/'
			);
		}
		else { $ewd_uasp_custom_fields_permissions = array(); }

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-custom-fields-tab',
				'title'         => __( 'Custom Fields', 'ultimate-appointment-scheduling' ),
				'is_tab'		=> true,
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-custom-fields',
					'title'         => __( 'Custom Fields', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-custom-fields-tab',
				),
				$ewd_uasp_custom_fields_permissions
			)
		);

		$fields_description = __( 'Should any extra fields be added to the booking form?', 'ultimate-appointment-scheduling' ) . '<br />';
		$fields_description .= __( 'The "Field Values" should be a comma-separated list of values for the select, radio or checkbox field types (no extra spaces after the comma).', 'ultimate-appointment-scheduling' ) . '<br />';
		
		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-custom-fields',
			'infinite_table',
			array(
				'id'			=> 'custom-fields',
				'title'			=> __( 'Custom Fields', 'ultimate-appointment-scheduling' ),
				'add_label'		=> __( '&plus; ADD', 'ultimate-appointment-scheduling' ),
				'del_label'		=> __( 'Delete', 'ultimate-appointment-scheduling' ),
				'description'	=> $fields_description,
				'fields'		=> array(
					'id' => array(
						'type' 		=> 'hidden',
						'label' 	=> 'Field ID',
						'required' 	=> true
					),
					'name' => array(
						'type' 		=> 'text',
						'label' 	=> 'Field Name',
						'required' 	=> true
					),
					
					'slug' => array(
						'type' 		=> 'text',
						'label' 	=> 'Field Slug',
						'required' 	=> true
					),
					'type' => array(
						'type' 		=> 'select',
						'label' 	=> __( 'Field Type', 'ultimate-appointment-scheduling' ),
						'options' 	=> array(
							'text'		=> __( 'Text', 'ultimate-appointment-scheduling' ),
							'number'	=> __( 'Number', 'ultimate-appointment-scheduling' ),
							'textarea'	=> __( 'Text Area', 'ultimate-appointment-scheduling' ),
							'select'	=> __( 'Select Box', 'ultimate-appointment-scheduling' ),
							'radio'		=> __( 'Radio Buttons', 'ultimate-appointment-scheduling' ),
							'checkbox'	=> __( 'Checkbox', 'ultimate-appointment-scheduling' ),
							'link'		=> __( 'Link', 'ultimate-appointment-scheduling' ),
							'date'		=> __( 'Date', 'ultimate-appointment-scheduling' ),
							'datetime'	=> __( 'Date/Time', 'ultimate-appointment-scheduling' ),
						)
					),
					'options' => array(
						'type' 		=> 'text',
						'label' 	=> __( 'Field Values', 'ultimate-appointment-scheduling' ),
						'required' 	=> false
					)
				)
			)
		);

		if ( ! $ewd_uasp_controller->permissions->check_permission( 'woocommerce' ) ) {
			$ewd_uasp_woocommerce_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> '#',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-appointment-scheduling/'
			);
		}
		else { $ewd_uasp_woocommerce_permissions = array(); }

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-payment-tab',
				'title'         => __( 'Payment', 'ultimate-appointment-scheduling' ),
				'is_tab'		=> true,
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-payment-woocommerce',
					'title'         => __( 'WooCommerce Payment Options', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-payment-tab',
				),
				$ewd_uasp_woocommerce_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-payment-woocommerce',
			'toggle',
			array(
				'id'			=> 'woocommerce-integration',
				'title'			=> __( 'Require WooCommerce Payment', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Should services be synced as WooCommerce products, and customers required to checkout before scheduling an appointment?', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-payment-paypal',
					'title'         => __( 'PayPal Payment Options', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-payment-tab',
				),
				$ewd_uasp_woocommerce_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-payment-paypal',
			'radio',
			array(
				'id'			=> 'paypal-prepayment',
				'title'			=> __( 'Allow Prepayment through PayPal', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Should customers be required to, able to (optional) or unable to pay for their appointments through PayPal when they book them?', 'ultimate-appointment-scheduling' ),
				'options'		=> array(
					'required'		=> __( 'Required', 'ultimate-appointment-scheduling' ),
					'optional'		=> __( 'Optional', 'ultimate-appointment-scheduling' ),
					'none'			=> __( 'None', 'ultimate-appointment-scheduling' ),
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-payment-paypal',
			'select',
			array(
				'id'            => 'currency-code',
				'title'         => __( 'Pricing Currency', 'ultimate-appointment-scheduling' ),
				'description'   => __( 'What currency are your services priced in?', 'ultimate-appointment-scheduling' ), 
				'blank_option'	=> false,
				'options'       => $this->currency_options
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-payment-paypal',
			'text',
			array(
				'id'            => 'paypal-email-address',
				'title'         => __( 'PayPal Email Address', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'If PayPal payments are required or optional, what email address is associated with the PayPal account?', 'ultimate-appointment-scheduling' ),
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-payment-paypal',
			'text',
			array(
				'id'            => 'thank-you-url',
				'title'         => __( '"Thank You" Page URL', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'What page should customers be taken to after successfully completing a PayPal payment?', 'ultimate-appointment-scheduling' ),
			)
		);

		if ( ! $ewd_uasp_controller->permissions->check_permission( 'reminders' ) ) {
			$ewd_uasp_reminders_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> '#',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-appointment-scheduling/'
			);
		}
		else { $ewd_uasp_reminders_permissions = array(); }

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-reminders-tab',
				'title'         => __( 'Reminders', 'ultimate-appointment-scheduling' ),
				'is_tab'		=> true,
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-reminders',
					'title'         => __( 'Reminder Options', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-reminders-tab',
				),
				$ewd_uasp_reminders_permissions
			)
		);

		$reminders_description = __( 'Should reminders be sent about appointments?', 'ultimate-appointment-scheduling' ) . '<br />';
		$reminders_description .= __( 'Note: Times are approximate, and depend on site traffic.', 'ultimate-appointment-scheduling' ) . '<br />';

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-reminders',
			'infinite_table',
			array(
				'id'			=> 'email-reminders',
				'title'			=> __( 'Appointment Email Reminders', 'ultimate-appointment-scheduling' ),
				'add_label'		=> __( '&plus; ADD', 'ultimate-appointment-scheduling' ),
				'del_label'		=> __( 'Delete', 'ultimate-appointment-scheduling' ),
				'description'	=> $reminders_description,
				'fields'		=> array(
					'id' => array(
						'type' 		=> 'hidden',
						'label' 	=> 'Reminder ID',
						'required' 	=> true
					),
					'interval' => array(
						'type' 		=> 'text',
						'label' 	=> 'Interval',
						'required' 	=> true
					),
					'unit' => array(
						'type' 		=> 'select',
						'label' 	=> __( 'Unit', 'ultimate-appointment-scheduling' ),
						'options' 	=> array(
							'minutes'	=> __( 'Minute(s)', 'ultimate-appointment-scheduling' ),
							'hours'		=> __( 'Hour(s)', 'ultimate-appointment-scheduling' ),
							'days'		=> __( 'Day(s)', 'ultimate-appointment-scheduling' ),
						)
					),
					'email_id' => array(
						'type' 		=> 'select',
						'label' 	=> __( 'Email', 'ultimate-appointment-scheduling' ),
						'options' 	=> $this->email_options
					),
					'conditional' => array(
						'type' 		=> 'select',
						'label' 	=> __( 'Ignore Confirmed?', 'ultimate-appointment-scheduling' ),
						'options' 	=> array(
							'no'		=> __( 'No', 'ultimate-appointment-scheduling' ),
							'yes'		=> __( 'Yes', 'ultimate-appointment-scheduling' ),
						)
					),
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-reminders',
			'toggle',
			array(
				'id'			=> 'appointment-confirmation',
				'title'			=> __( 'Appointment Confirmation', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Should a link be included in reminder emails so that appointments can be confirmed? (Also requires filling out the "Appointment Booking Page" field below)', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-reminders',
			'text',
			array(
				'id'            => 'appointment-booking-page',
				'title'         => __( 'Appointment Booking Page', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'What\'s the URL of the page with the appointment booking form, if "Appointment Confirmation" is set to "Yes"?', 'ultimate-appointment-scheduling' ),
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-reminders',
			'count',
			array(
				'id'			=> 'reminders-cache-time',
				'title'			=> __( 'Reminders Cache Time', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'How often should the plugin check to see if there are new reminders to send out when a page is loaded?', 'ultimate-appointment-scheduling' ),
				'blank_option'	=> false,
				'min_value'		=> 1,
				'max_value'		=> 90,
				'increment'		=> 1,
				'units'			=> array( 'minutes' => 'Minutes' )
			)
		);

		if ( ! $ewd_uasp_controller->permissions->check_permission( 'emails' ) ) {
			$ewd_uasp_emails_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> '#',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-appointment-scheduling/'
			);
		}
		else { $ewd_uasp_emails_permissions = array(); }

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-emails-tab',
				'title'         => __( 'Emails', 'ultimate-appointment-scheduling' ),
				'is_tab'		=> true,
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-emails',
					'title'         => __( 'Email Messages', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-emails-tab',
				),
				$ewd_uasp_emails_permissions
			)
		);

		$email_messages_description = '<ul><li>';
		$email_messages_description .= __( 'Use the table above to build emails for your users.', 'ultimate-appointment-scheduling' );
		$email_messages_description .= '</li><li>';
		$email_messages_description .= __( 'You can use [section]...[/section] and [footer]...[/footer] to split up the content of your email. You can also include a link button, like so: [button link=\'LINK_URL_GOES_HERE\']BUTTON_TEXT[/button], and a link to review each individual item in an order with:[review-items link=\'LINK_URL_TO_SUBMIT_REVIEW_PAGE\']', 'ultimate-appointment-scheduling' );
		$email_messages_description .= '</li><li>';
		$email_messages_description .= __( 'You can also put [appointment-time], [client], [phone], [email], [location], [service], [service-provider], [cancellation-link] or [confirmation-link] (if "Appointment Confirmation" is enabled) into the message body or subject.', 'ultimate-appointment-scheduling' );
		$email_messages_description .= '</li><li>';
		$email_messages_description .= __( 'Use the area below to send yourself a sample email.', 'ultimate-appointment-scheduling' );
		$email_messages_description .= '</li></ul>';

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-emails',
			'infinite_table',
			array(
				'id'			=> 'email-messages-array',
				'title'			=> __( 'Email Messages', 'ultimate-appointment-scheduling' ),
				'add_label'		=> __( '&plus; ADD', 'ultimate-appointment-scheduling' ),
				'del_label'		=> __( 'Delete', 'ultimate-appointment-scheduling' ),
				'description'	=> $email_messages_description,
				'fields'		=> array(
					'id' => array(
						'type' 		=> 'hidden',
						'label' 	=> 'ID',
						'required' 	=> true
					),
					'name' => array(
						'type' 		=> 'text',
						'label' 	=> __( 'Email Name', 'ultimate-appointment-scheduling' ),
						'required' 	=> true
					),
					'subject' => array(
						'type' 		=> 'text',
						'label' 	=> __( 'Message Subject', 'ultimate-appointment-scheduling' ),
						'required' 	=> true
					),
					'message' => array(
						'type' 		=> 'textarea',
						'label' 	=> __( 'Message', 'ultimate-appointment-scheduling' ),
						'required' 	=> true
					)
				)
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-emails',
			'select',
			array(
				'id'            => 'send-sample-email-message',
				'title'         => __( 'Sample Email Message', 'ultimate-appointment-scheduling' ),
				'description'	=> __( 'Choose an email message to send as a sample.', 'ultimate-appointment-scheduling' ),
				'blank_option'	=> false,
				'options'       => $this->email_options
			)
		);

		$sample_email_address_description = __( 'Choose an email address to send the above sample email message to. Make sure that you click the "Save Changes" button below before sending the test message, to receive the most recent version of your email.', 'ultimate-appointment-scheduling' );
		$sample_email_address_description .= '<br><br>';
		$sample_email_address_description .= '<button type="button" class="ewd-uasp-send-test-email">Send Sample Email</button>';

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-emails',
			'text',
			array(
				'id'            => 'send-sample-email-address',
				'title'         => __( 'Sample Email Address', 'ultimate-appointment-scheduling' ),
				'description'	=> $sample_email_address_description
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-email-styling',
					'title'         => __( 'Reminder Email Styling', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-emails-tab',
				),
				$ewd_uasp_emails_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-email-styling',
			'colorpicker',
			array(
				'id'			=> 'styling-email-background-color',
				'title'			=> __( 'Email Background Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-email-styling',
			'colorpicker',
			array(
				'id'			=> 'styling-email-inner-color',
				'title'			=> __( 'Email Inner Background Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-email-styling',
			'colorpicker',
			array(
				'id'			=> 'styling-email-text-color',
				'title'			=> __( 'Email Text Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-email-styling',
			'colorpicker',
			array(
				'id'			=> 'styling-email-button-bg-color',
				'title'			=> __( 'Email Button Background Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-email-styling',
			'colorpicker',
			array(
				'id'			=> 'styling-email-button-text-color',
				'title'			=> __( 'Email Button Text Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-email-styling',
			'colorpicker',
			array(
				'id'			=> 'styling-email-button-bg-hover-color',
				'title'			=> __( 'Email Button Hover Background Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-email-styling',
			'colorpicker',
			array(
				'id'			=> 'styling-email-button-text-hover-color',
				'title'			=> __( 'Email Button Hover Text Color', 'ultimate-appointment-scheduling' )
			)
		);

		if ( ! $ewd_uasp_controller->permissions->check_permission( 'labelling' ) ) {
			$ewd_uasp_labelling_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> '#',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-appointment-scheduling/'
			);
		}
		else { $ewd_uasp_labelling_permissions = array(); }

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-labelling-tab',
				'title'         => __( 'Labelling', 'ultimate-appointment-scheduling' ),
				'is_tab'		=> true,
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-labelling-sign-up',
					'title'         => __( 'Sign Up', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-labelling-tab',
				),
				$ewd_uasp_labelling_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-sign-up',
			'text',
			array(
				'id'            => 'label-sign-up-title',
				'title'         => __( '\'Sign Up\' Title', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-sign-up',
			'text',
			array(
				'id'            => 'label-name',
				'title'         => __( 'Name', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-sign-up',
			'text',
			array(
				'id'            => 'label-phone',
				'title'         => __( 'Phone', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-sign-up',
			'text',
			array(
				'id'            => 'label-email',
				'title'         => __( 'Email', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-labelling-service',
					'title'         => __( 'Service', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-labelling-tab',
				),
				$ewd_uasp_labelling_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-service',
			'text',
			array(
				'id'            => 'label-service-title',
				'title'         => __( '\'Service\' Title', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-service',
			'text',
			array(
				'id'            => 'label-location',
				'title'         => __( 'Location', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-service',
			'text',
			array(
				'id'            => 'label-service',
				'title'         => __( 'Service', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-service',
			'text',
			array(
				'id'            => 'label-service-provider',
				'title'         => __( 'Service Provider', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-service',
			'text',
			array(
				'id'            => 'label-any',
				'title'         => __( 'Any (Service Provider)', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-labelling-appointment',
					'title'         => __( 'Appointment', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-labelling-tab',
				),
				$ewd_uasp_labelling_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-appointment-title',
				'title'         => __( '\'Appointment\' Title', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-appointment-date',
				'title'         => __( 'Appointment Date', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-find-appointment',
				'title'         => __( 'Find Appointments', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-book-appointment',
				'title'         => __( 'Book Appointment', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-pay-in-advance',
				'title'         => __( 'Pay in Advance', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-proceed-to-payment',
				'title'         => __( 'Proceed to Payment', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-select-time',
				'title'         => __( 'Select Time', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-click-select-date',
				'title'         => __( 'Click here to select a date', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-labelling-appointment',
			'text',
			array(
				'id'            => 'label-image-number',
				'title'         => __( 'Image Number', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		if ( ! $ewd_uasp_controller->permissions->check_permission( 'styling' ) ) {
			$ewd_uasp_styling_permissions = array(
				'disabled'		=> true,
				'disabled_image'=> '#',
				'purchase_link'	=> 'https://www.etoilewebdesign.com/plugins/ultimate-appointment-scheduling/'
			);
		}
		else { $ewd_uasp_styling_permissions = array(); }

		$sap->add_section(
			'ewd-uasp-settings',
			array(
				'id'            => 'ewd-uasp-styling-tab',
				'title'         => __( 'Styling', 'ultimate-appointment-scheduling' ),
				'is_tab'		=> true,
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-styling-sign-up',
					'title'         => __( 'Sign Up Form', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-styling-tab',
				),
				$ewd_uasp_styling_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-sign-up',
			'colorpicker',
			array(
				'id'			=> 'styling-signup-title-font-color',
				'title'			=> __( 'Title Font Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-sign-up',
			'text',
			array(
				'id'            => 'styling-signup-title-font',
				'title'         => __( 'Title Font Family', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-sign-up',
			'text',
			array(
				'id'            => 'styling-signup-title-font-size',
				'title'         => __( 'Title Font Size', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-sign-up',
			'text',
			array(
				'id'            => 'styling-signup-label-font',
				'title'         => __( 'Label Font Family', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-sign-up',
			'text',
			array(
				'id'            => 'styling-signup-label-font-size',
				'title'         => __( 'Label Font Size', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-sign-up',
			'colorpicker',
			array(
				'id'			=> 'styling-signup-block-color',
				'title'			=> __( 'Block Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-sign-up',
			'text',
			array(
				'id'            => 'styling-signup-block-margin',
				'title'         => __( 'Block Margin', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-sign-up',
			'text',
			array(
				'id'            => 'styling-signup-block-padding',
				'title'         => __( 'Block Padding', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-styling-service',
					'title'         => __( 'Choose Service Form', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-styling-tab',
				),
				$ewd_uasp_styling_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-service',
			'colorpicker',
			array(
				'id'			=> 'styling-service-title-font-color',
				'title'			=> __( 'Title Font Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-service',
			'text',
			array(
				'id'            => 'styling-service-title-font',
				'title'         => __( 'Title Font Family', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-service',
			'text',
			array(
				'id'            => 'styling-service-title-font-size',
				'title'         => __( 'Title Font Size', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-service',
			'text',
			array(
				'id'            => 'styling-service-label-font',
				'title'         => __( 'Label Font Family', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-service',
			'text',
			array(
				'id'            => 'styling-service-label-font-size',
				'title'         => __( 'Label Font Size', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-service',
			'colorpicker',
			array(
				'id'			=> 'styling-service-block-color',
				'title'			=> __( 'Block Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-service',
			'text',
			array(
				'id'            => 'styling-service-block-margin',
				'title'         => __( 'Block Margin', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-service',
			'text',
			array(
				'id'            => 'styling-service-block-padding',
				'title'         => __( 'Block Padding', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-styling-appointment',
					'title'         => __( 'Find Appointment Form', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-styling-tab',
				),
				$ewd_uasp_styling_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-appointment',
			'colorpicker',
			array(
				'id'			=> 'styling-appointment-title-font-color',
				'title'			=> __( 'Title Font Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-appointment',
			'text',
			array(
				'id'            => 'styling-appointment-title-font',
				'title'         => __( 'Title Font Family', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-appointment',
			'text',
			array(
				'id'            => 'styling-appointment-title-font-size',
				'title'         => __( 'Title Font Size', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-appointment',
			'text',
			array(
				'id'            => 'styling-appointment-label-font',
				'title'         => __( 'Label Font Family', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-appointment',
			'text',
			array(
				'id'            => 'styling-appointment-label-font-size',
				'title'         => __( 'Label Font Size', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-appointment',
			'colorpicker',
			array(
				'id'			=> 'styling-appointment-block-color',
				'title'			=> __( 'Block Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-appointment',
			'text',
			array(
				'id'            => 'styling-appointment-block-margin',
				'title'         => __( 'Block Margin', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-appointment',
			'text',
			array(
				'id'            => 'styling-appointment-block-padding',
				'title'         => __( 'Block Padding', 'ultimate-appointment-scheduling' ),
				'description'	=> ''
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-styling-calendar',
					'title'         => __( 'Calendar Styling', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-styling-tab',
				),
				$ewd_uasp_styling_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-calendar',
			'text',
			array(
				'id'			=> 'styling-calendar-appt-font',
				'title'			=> __( 'Appointment Font Family', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-calendar',
			'text',
			array(
				'id'			=> 'styling-calendar-appt-font-size',
				'title'			=> __( 'Appointment Font Size', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-calendar',
			'colorpicker',
			array(
				'id'			=> 'styling-calendar-appt-color',
				'title'			=> __( 'Appointment Font Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-calendar',
			'colorpicker',
			array(
				'id'			=> 'styling-calendar-appt-bg-color',
				'title'			=> __( 'Appointment Background Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-calendar',
			'colorpicker',
			array(
				'id'			=> 'styling-calendar-appt-border-color',
				'title'			=> __( 'Appointment Border Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-calendar',
			'colorpicker',
			array(
				'id'			=> 'styling-calendar-appt-selected-bg-color',
				'title'			=> __( 'Selected Appointment Background Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-calendar',
			'colorpicker',
			array(
				'id'			=> 'styling-calendar-appt-selected-border-color',
				'title'			=> __( 'Selected Appointment Border Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_section(
			'ewd-uasp-settings',
			array_merge(
				array(
					'id'            => 'ewd-uasp-styling-button',
					'title'         => __( 'Button Styling', 'ultimate-appointment-scheduling' ),
					'tab'	        => 'ewd-uasp-styling-tab',
				),
				$ewd_uasp_styling_permissions
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-button',
			'colorpicker',
			array(
				'id'			=> 'styling-button-font-color',
				'title'			=> __( 'Text Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-button',
			'colorpicker',
			array(
				'id'			=> 'styling-button-color',
				'title'			=> __( 'Background Color', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-button',
			'text',
			array(
				'id'			=> 'styling-button-font',
				'title'			=> __( 'Font Family', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-button',
			'text',
			array(
				'id'			=> 'styling-button-font-size',
				'title'			=> __( 'Font Size', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-button',
			'text',
			array(
				'id'			=> 'styling-button-margin',
				'title'			=> __( 'Margin', 'ultimate-appointment-scheduling' )
			)
		);

		$sap->add_setting(
			'ewd-uasp-settings',
			'ewd-uasp-styling-button',
			'text',
			array(
				'id'			=> 'styling-button-padding',
				'title'			=> __( 'Padding', 'ultimate-appointment-scheduling' )
			)
		);

		$sap = apply_filters( 'ewd_uasp_settings_page', $sap );

		$sap->add_admin_menus();

	}
}
} // endif;
